<?php
/**
 * Plugin Name: Clickbank Framework
 * Plugin URI: http://memberbuddy.com/docs/clickbank-framework
 * Description: Clickbank integration framework
 * Version: 0.0.1
 * Author: Rob Holmes
 * Author URI: http://memberbuddy.com/people
 */

/*  Copyright 2011 Rob Holmes ( email: rob@onemanonelaptop.com )

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
class wpClickbankFramework {

	// Force singelton
	static $instance = false;
	public static function getInstance() {
		return (self::$instance ? self::$instance : self::$instance = new self);
	}
	
	// Constructor
	private function __construct() {
		// Register the options on the general settings page
		add_action( 'admin_init', array($this,'clickbank_register_settings') );
	
		// Assign the query variables and the handlers
		add_action( 'template_redirect', array( $this, 'receive_clickbank_ipn' ));
		add_filter( 'query_vars', array( $this, 'clickbank_listener' ));
	
		// Register the activation hook
		register_activation_hook(__FILE__, array( $this, 'plugin_activate' ) );
	} // function
		
	// Register some options on the general settings page
	function clickbank_register_settings() {
		add_settings_section('clickbank_setting_section', 'Clickbank IPN Settings', array(&$this,'clickbank_settings_section_callback'), 'general');
		add_settings_field('clickbank-key',	'Secret Key', array(&$this,'clickbank_settings_field_key'), 'general', 'clickbank_setting_section');
		add_settings_field('clickbank-debug',	'Debug Email Address', array(&$this,'clickbank_settings_field_debug'), 'general', 'clickbank_setting_section');
		register_setting('general','clickbank-key');
		register_setting('general','clickbank-debug');
	} // function
 
	// Section callback 
	function clickbank_settings_section_callback() { 
		print 'Your Clickbank Instant Payment Notification URL is <span style="color:red;">' . site_url() . '?clickbank=IPN</span>';
	} // function
	 
	// List the checkboxes
	function clickbank_settings_field_key() {	
		$key = get_option('clickbank-key');
		print '<input type="text" style="" name="clickbank-key" placeholder="Enter your secret key..." value="' . $key . '" /><span class="description"> Clickbank Secret Key</span><br/>';
	} // function
	
	// List the checkboxes
	function clickbank_settings_field_debug() {	
		$debug = get_option('clickbank-debug');
		print '<input type="text" style="width:300px;" name="clickbank-debug[address]" placeholder="Enter an email address"  value="' . $debug['address']. '" />';
		print '<input type="checkbox" name="clickbank-debug[enable]"  value="1" ' . checked( 1, $debug['enable'], false ) . ' /><span class="description"> Enable Debugging emails</span><br/>';
	} // function

	public function clickbank_listener($public_query_vars) {
		$public_query_vars[] = 'clickbank';
		return $public_query_vars;
	} // function
	
	// Setup the lister for the clickbank IPN and do stuff when it happens
	public function receive_clickbank_ipn() {
		
		// Check that the query var is set and is the correct value.
		if (get_query_var( 'clickbank' ) == 'IPN') {
			$_POST = stripslashes_deep($_POST);
			
			// Try to validate the response to make sure it's from clickbank
			if ($this->clickbank_ipn_verification($_POST)) {
				$this->process_clickbank_message();
			}else {
			    // Stop WordPress entirely
				$this->debug_email('Clickbank IPN Debug - Validation Fail','',$_POST);
			}
			exit;
		}
	} // function
	
	// Verify the IPN
	function clickbank_ipn_verification($postdata) {
		$secretKey = get_option('clickbank-key');
		$pop = "";
		$ipnFields = array();
		foreach ($postdata as $key => $value) {
			if ($key == "cverify") {
				continue;
			}
			$ipnFields[] = $key;
		}
		sort($ipnFields);
		foreach ($ipnFields as $field) {
			// if Magic Quotes are enabled $_POST[$field] will need to be
			// un-escaped before being appended to $pop
			$pop = $pop . $postdata[$field] . "|";
		}
		$pop = $pop . $secretKey;
		$calcedVerify = sha1(mb_convert_encoding($pop, "UTF-8"));
		$calcedVerify = strtoupper(substr($calcedVerify,0,8));
	
		return $calcedVerify == $postdata["cverify"];
	} // function
		 			
					
	// Clickbank : Throw an action based off the transaction type of the message - Based on Paypal Framework by Aaron Cambell (Xavisys)
	private function process_clickbank_message() {
		do_action("clickbank-ipn", $_POST);
		if ( !empty($_POST['ctransaction']) ) {
			$specificAction = " and clickbank-{$_POST['ctransaction']}";
			do_action("clickbank-{$_POST['ctransaction']}", $_POST);
		}
		$this->debug_email('Clickbank IPN Debug - Message Processed', "Actions thrown: clickbank-ipn{$specificAction}\r\n\r\n",$_POST);
	} // function	
		
	// Helper function to send out the debugging email
	private function debug_email($subject,$message = '',$post = '') {
		$debug = get_option('clickbank-debug');
		if ( $debug['enable'] == '1' && !empty($debug['address'] ) ) {
			wp_mail($debug['address'] , $subject, $message ."\r\n\r\nPost Data:\r\n\r\n".print_r($post, true));
		}
	} // function
	
}// class
	
// Instantiate
$wpClickbankFramework = wpClickbankFramework::getInstance();

?>